from random import choice

class MarkovGenerator(object):

    """A token generator using a Markov chain with configurable order.
    
    Queneau assembly is usually better than a Markov chain above the
    word level (constructing paragraphs from sentences) and below the word
    level (constructing words from phonemes), but Markov chains are
    usually better when assembling sequences of words.
    """

    def __init__(self, order=1, max=500):
        self.order = order # order (length) of ngrams
        self.max = max # maximum number of elements to generate
        self.ngrams = dict() # ngrams as keys; next elements as values
        self.beginnings = list() # beginning ngram of every line

    @classmethod
    def load(cls, f, order=1, max=500):
        """Load from a filehandle that defines a single chunk of text."""
        corpus = MarkovGenerator(order, max)
        corpus.add(f.read())
        return corpus

    @classmethod
    def loadlines(cls, f, order=1, max=500):
        """Load from a filehandle that defines one text per line."""
        corpus = MarkovGenerator(order, max)
        if not hasattr(f, 'read'):
            # Not a file-type object. Treat it as a multi-line string.
            f = f.split("\n")
        for l in f:
            corpus.add(l.strip())
        return corpus

    def tokenize(self, text):
        return text.split(" ")

    def add(self, text):
        tokens = self.tokenize(text)
        # discard this line if it's too short
        if len(tokens) < self.order:
            return

        # store the first ngram of this line
        beginning = tuple(tokens[:self.order])
        self.beginnings.append(beginning)

        i = 0
        for i in range(len(tokens) - self.order):

            gram = tuple(tokens[i:i+self.order])
            next = tokens[i+self.order] # get the element after the gram

            # if we've already seen this ngram, append; otherwise, set the
            # value for this key as a new list
            self.ngrams.setdefault(gram, []).append(next)

        if i > 0:
            # Store the fact that a given token was the last one on the line.
            final_gram = tuple(tokens[i+1:i+self.order+1])
            self.ngrams.setdefault(final_gram, []).append(None)

    # called from generate() to join together generated elements
    def concatenate(self, source):
        return " ".join(source)

    def assemble(self):
        "Yield a new text similar to existing texts."

        # get a random line beginning; convert to a list. 
        current = choice(self.beginnings)
        output = list(current)
        for token in current:
            yield token

        done = False
        for i in range(self.max):
            if current in self.ngrams:
                possible_next = self.ngrams[current]
                next = choice(possible_next)
                if next is None:
                    # This is the final item!
                    done = True
                    break
                yield self.modify(next)
                output.append(next)
                # get the last N entries of the output; we'll use this to look up
                # an ngram in the next iteration of the loop
                current = tuple(output[-self.order:])
            else:
                break

    generate = assemble
    chain = assemble

    def modify(self, token):
        """Modify a token before yielding it."""
        return token

class BracketMatchingMarkovGenerator(MarkovGenerator):

    """A generator that tries to ensure balanced brackets and double quotes.

    It's not perfect, but it's a lot better than nothing.
    """

    def __init__(self, *args, **kwargs):
        super(BracketMatchingMarkovGenerator, self).__init__(*args, **kwargs)
        self.useful_tokens = set([])

    def tokenize(self, text):
        tokens = super(BracketMatchingMarkovGenerator, self).tokenize(text)
        for token in tokens:
            for closing in ['"', ")", "]", "}"]:
                if token.endswith(closing):
                    self.useful_tokens.add(token)
        return tokens

    def assemble(self):
        self.stack = []
        for x in super(BracketMatchingMarkovGenerator, self).assemble():
            yield x

    def modify(self, token_to_yield):
        # Is there an opening bracket in this token?
        for opening, closing in ['""', "()", "[]", "{}"]:
            if opening in token_to_yield:
                index = token_to_yield.find(opening)
                if token_to_yield.find(closing, index+1) == -1:
                    self.stack.append(closing)
            elif closing in token_to_yield:
                # We are closing a bracket. Is there an open bracket?
                if len(self.stack) == 0:
                    #Nope.
                    token_to_yield = token_to_yield.replace(closing, "")
                elif closing != self.stack[-1]:
                    # Force it to be the right kind of bracket.
                    token_to_yield = token_to_yield.replace(
                        closing, self.stack[-1], 1)
                    self.stack.pop()
                else:
                    # It's already the right kind of bracket.
                    self.stack.pop()

        if len(self.stack) > 0:
            # If we modify this token by appending the closing bracket
            # on top of the stack, would we get a token that exists in
            # the corpus?
            check_for = token_to_yield + self.stack[-1]
            if check_for in self.useful_tokens:
                self.stack.pop()
                return check_for
        # No luck. Return the original token.
        return token_to_yield
  
if __name__ == '__main__':
    import sys
    generator = MarkovGenerator.loadlines(sys.stdin, order=1, max=500)
    for i in range(14):
        print(" ".join(list(generator.assemble())))



from __future__ import absolute_import
import re
from textblob.base import BaseTokenizer

class WordTokenizer(BaseTokenizer):
    """
This tokenizer is copy-pasted version of TreebankWordTokenizer
that doesn't split on @ and ':' symbols and doesn't split contractions::

>>> from nltk.tokenize.treebank import TreebankWordTokenizer
>>> s = '''Good muffins cost $3.88\\nin New York. Email: muffins@gmail.com'''
>>> TreebankWordTokenizer().tokenize(s)
['Good', 'muffins', 'cost', '$', '3.88', 'in', 'New', 'York.', 'Email', ':', 'muffins', '@', 'gmail.com']
>>> WordTokenizer().tokenize(s)
['Good', 'muffins', 'cost', '$', '3.88', 'in', 'New', 'York.', 'Email:', 'muffins@gmail.com']

>>> s = '''Shelbourne Road,'''
>>> WordTokenizer().tokenize(s)
['Shelbourne', 'Road', ',']

>>> s = '''population of 100,000'''
>>> WordTokenizer().tokenize(s)
['population', 'of', '100,000']
"""
    def tokenize(self, text):
        # starting quotes
        text = re.sub(r'^\"', r'``', text)
        text = re.sub(r'(``)', r' \1 ', text)
        text = re.sub(r'([ (\[{<])"', r'\1 `` ', text)

        # punctuation
        text = re.sub(r'(,)(\D|\Z)', r' \1 \2', text)
        text = re.sub(r'\.\.\.', r' ... ', text)
        text = re.sub(r'[;#$%&]', r' \g<0> ', text) # CHANGED @


        text = re.sub(r'([^\.])(\.)([\]\)}>"\']*)\s*$', r'\1 \2\3 ', text)
        text = re.sub(r'[?!]', r' \g<0> ', text)

        text = re.sub(r"([^'])' ", r"\1 ' ", text)

        # parens, brackets, etc.
        text = re.sub(r'[\]\[\(\)\{\}\<\>]', r' \g<0> ', text)
        text = re.sub(r'--', r' -- ', text)

        # add extra space to make things easier
        text = " " + text + " "

        #ending quotes
        text = re.sub(r'"', " '' ", text)
        text = re.sub(r'(\S)(\'\')', r'\1 \2 ', text)

        # CHANGED:

        # text = re.sub(r"([^' ])('[sS]|'[mM]|'[dD]|') ", r"\1 \2 ", text)
        # text = re.sub(r"([^' ])('ll|'LL|'re|'RE|'ve|'VE|n't|N'T) ", r"\1 \2 ",
        # text)
        #
        # for regexp in self.CONTRACTIONS2:
        # text = regexp.sub(r' \1 \2 ', text)
        # for regexp in self.CONTRACTIONS3:
        # text = regexp.sub(r' \1 \2 ', text)

        # We are not using CONTRACTIONS4 since
        # they are also commented out in the SED scripts
        # for regexp in self.CONTRACTIONS4:
        # text = regexp.sub(r' \1 \2 \3 ', text)

        return text.split()

    def itokenize(self, text, *args, **kwargs):
        return iterator(self.tokenize(text))

word_tokenizer = WordTokenizer()

def default_tokenizer(text):
    for tok in word_tokenizer.tokenize(text):
        if tok in ',;':
            continue
        yield tok




#!/usr/bin/env python
import sys
from io import open

import setuptools

requires = ['textblob', 'wordfilter', 'internetarchive', 'requests']

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name='olipy',
    version='1.0.4',
    author='Leonard Richardson',
    author_email='leonardr@segfault.org',
    url="https://github.com/leonardr/olipy/",
    description="Python library for artistic text generation",
    license='GPLv3',
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),
    install_requires=requires,
    entry_points={
        'console_scripts': [
            'olipy.apollo = olipy.example:apollo',
            'olipy.board_games = olipy.example:board_games',
            'olipy.corrupt = olipy.example:corrupt',
            'olipy.dinosaurs = olipy.example:dinosaurs',
            'olipy.ebooks = olipy.example:ebooks',
            'olipy.gibberish = olipy.example:gibberish',
            'olipy.mashteroids = olipy.example:mashteroids',
            'olipy.sonnet = olipy.example:sonnet',
            'olipy.typewriter = olipy.example:typewriter',
            'olipy.words = olipy.example:words',
        ]
    },
    package_data = {
        "olipy": [
            "data/%s/*.json" % ("*/" * x)
            for x in range(10)
        ]
    },
    classifiers=[
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: Implementation :: CPython',
        'Topic :: Text Processing',
        'Topic :: Artistic Software',
    ],
)



"""Code for dealing with the Internet Archive."""
import datetime
import internetarchive as ia
import requests
try:
    import urlparse
except ImportError:
    import urllib.parse as urlparse

class Item(object):
    "Wraps the ia.item class with extra utilities."""

    MEDIA_TYPE = None
    DATE_FORMAT = "%Y-%m-%dT%H:%M:%SZ"

    _session = None

    def __init__(self, identifier):
        item = metadata = None
        if isinstance(identifier, ia.item.Item):
            # This is an actual Item object from the underlying
            # API wrapper.
            item = identifier
            identifier = item.identifier
            metadata = item.metadata
        elif isinstance(identifier, dict):
            # This is raw metadata from the underlying API wrapper
            item = None
            metadata = identifier
            identifier = identifier['identifier']
        self.identifier = identifier
        self._item = item
        self._metadata = metadata
        self._files = None

    @classmethod
    def session(cls, set_to=None):
        """Keep one ArchiveSession object for the whole program.

        The session doesn't seem to keep any state so this should be
        fine.
        """
        if set_to:
            cls._session = set_to
        if not cls._session:
            cls._session = ia.session.ArchiveSession()
        return cls._session

    @classmethod
    def recent(cls, query="", cutoff=None, fields=None, sorts=None, page=100, *args, **kwargs):
        """Find all the items that match `query` that were added since
        `date`.
        """
        if isinstance(cutoff, basestring):
            cutoff = datetime.datetime.strptime(cutoff, cls.DATE_FORMAT)
        sorts = sorts or []
        sorts.insert(0, 'publicdate desc')

        fields = fields or []
        for item in cls.search(query, *args, fields=fields, sorts=sorts, **kwargs):
            if cutoff and item.date('publicdate') < cutoff:
                break
            yield item


    @classmethod
    def search(cls, query, collection=None, fields=None, sorts=None, *args, **kwargs):
        """Search Internet Archive items.
        :param fields: Retrieve these metadata fields for each item. List of
            fields: https://archive.org/services/search/v1/fields
            By default, 'title', 'publidate', and 'identifier' are retrieved.
        :param sorts: A list of fields to use as sort order. Add
            " asc" or " desc" to the name of the field to specify
            ascending or descending order.
        """
        fields = fields or []
        for mandatory in ['title', 'publicdate']:
            if not mandatory in fields:
                fields.append(mandatory)
        query = cls.modified_query(query, collection)
        search = ia.search.Search(
            cls.session(), query, *args, fields=fields, sorts=sorts,
            **kwargs
        )
        for i in search.iter_as_results():
            yield cls(i)

    @classmethod
    def modified_query(cls, query, collection):
        addenda = []
        if collection:
            addenda.append("collection:%s" % collection)
        if cls.MEDIA_TYPE:
            addenda.append("mediatype:%s" % cls.MEDIA_TYPE)
        if not addenda:
            return query
        if query:
            query += ' and '
        extra = " and ".join(addenda)
        return query + extra

    def date(self, field="date"):
        """Parse a date field associated with this item."""
        if not field.endswith('date'):
            field += 'date'
        data = self.metadata.get(field)
        if not data:
            return None
        parsed = datetime.datetime.strptime(data, self.DATE_FORMAT)
        return parsed

    @property
    def item(self):
        if not self._item:
            self._item = ia.get_item(self.identifier)
        return self._item

    @property
    def files(self):
        if not self._files:
            self._files = list(self.item.get_files())
        return self._files

    @property
    def metadata(self):
        if not self._metadata:
            self._metadata = self.item.metadata
        return self._metadata


class Text(Item):
    """This class knows about the IA book reader."""

    MEDIA_TYPE = "texts"

    # The URL to a specific page in the IA book reader.
    reader_template = "https://archive.org/details/%(identifier)s/page/n%(page)d"

    # The URL to the actual image of a specific page used in the IA book reader.
    reader_image_template = "https://%(server)s/BookReader/BookReaderImages.php?zip=%(zip_path)s&file=%(image_path)s"

    # The path to a ZIP file on an IA server, used to fill in %(zip_path)s in
    # reader_image_template
    zip_path_template = "/%(directory_number)s/items/%(identifier)s/%(archive_filename)s"

    @property
    def pages(self):
        """How many pages are in this book?

        i.e. how many images are in this text?
        """
        return int(self.metadata.get('imagecount', 0))

    @property
    def jp2_url(self):
        """Find the URL to the JP2 version of this text.

        This is essential to finding working image URLs.
        """
        jp2 = [
            x for x in self.files
            if x.format == u'Single Page Processed JP2 ZIP'
            and x.exists
        ]
        if not jp2:
            return None
        return jp2[0].url

    def reader_url(self, page):
        """Generate the URL to the Internet Archive reader for page X."""
        return self.reader_template % dict(
            identifier=self.identifier,
            page=page
        )

    def image_url(self, page, **kwargs):
        """Generate the URL to an image for page X.

        Before using the image you might want to make a HEAD request
        to make sure the image is actually there. The URL generation
        works pretty reliably, but I've seen cases where the book is
        shorter than reported.

        :param kwargs: Will be appended to the URL as extra arguments.
        Useful arguments include 'scale' and 'rotate'.
        """

        # Get the URL to the JP2 version of the text. This is the
        # version used by the web reader, so it's important to know
        # what it's called.
        jp2_url = self.jp2_url

        # Make a HEAD request to the jp2 URL to see which server the
        # file is on and which directory it's in.
        response = requests.head(jp2_url)
        if response.status_code != 302 or not 'location' in response.headers:
            return None
        location = response.headers['location']
        parsed = urlparse.urlparse(location)

        # ZIP files are hosted at servers whose URLs look like this:
        # https://ia600106.us.archive.org/
        server = parsed.netloc

        path = parsed.path.split("/")
        directory_number = path[1]
        jp2_filename = path[-1]

        # The ZIP lives on the server at a path that looks like:
        # /30/items/identifier/A_Great_Book/A_Great_Book_jp2.zip
        zip_path = self.zip_path_template % dict(
            directory_number=directory_number,
            identifier=self.identifier,
            archive_filename=jp2_filename,
        )

        # ZIP files for texts contain one JP2 file per page, in a
        # directory named after the ZIP file:
        # A_Great_Book_jp2/A_Great_Book_0000.jp2
        with_format_identifier = jp2_filename[:-len('.zip')]
        filename_base = jp2_filename[:-len("_jp2.zip")]
        image_filename = filename_base + "_%.4d.jp2" % page
        path_within_file = "/".join([with_format_identifier, image_filename])

        image_url = self.reader_image_template % dict(
            server=server,
            zip_path=zip_path,
            image_path=path_within_file,
        )
        extra = "&".join("%s=%s" % (k, v) for k, v in kwargs.items())
        if extra:
            image_url += "&" + extra
        return image_url


class Audio(Item):
    """This class knows about audio items."""
    MEDIA_TYPE = "audio"



# encoding: utf-8
from olipy.randomness import WanderingMonsterTable
import random

class Typewriter(object):
    """Simulates the Adler Universal 39 typewriter used in "The Shining"
    and the sorts of typos that are commonly made on that typewriter.

    Originally written for @a_dull_bot.
    """
    neighbors = dict(
        A = "SQZW",
        l = "kop:,.",
        w = "q23esa",
        o = "90ipkl:",
        r = "45etdfg",
        k = "iojlm,",
        a = "sqzw",
        n = "bjkm ",
        d = "ersfxc",
        p = u"0-=o½l:",
        y = "67tughj",
        m = "jkln, ",
        e = "34wrsdf",
        s = "weadzx",
        J = "YUIHKNM",
        c = "dfxv ",
        u = "78yihj",
        b = "ghvn ",
    )
    neighbors[" "] = "    zxcvbnm,./"
    neighbors["."] = "l;,/"

    def __init__(b, mean_transforms=1.4, stdev_transforms=0.5):
        """Simulate someone hacking away at a typewriter

        :param mean_transforms: The mean number of mistakes per 100
        characters.

        :param stdev_stransforms: The acceptable standard
        deviation from the mean.
        """
        b.mean_transforms = mean_transforms
        b.stdev_transforms = stdev_transforms
        b.wmt = WanderingMonsterTable(
            common=[
                b.typo, b.typo, b.duplicate, b.transpose
            ],
            uncommon=[b.omit_period, b.delete_space, b.typo_add, b.delete],
            rare=[b.delete, b.lowercase_uppercase_letter,
                  b.extra_space_at_beginning, b.uppercase_letter],
            very_rare=[b.uppercase_word, b.uppercase_entire_string,
                       b.lowercase_entire_string, b.remove_word,
                       b.delete],
        )


    def find_typo(self, correct):
        if correct not in self.neighbors:
            # This is already a typo.
            return correct
        return random.choice(self.neighbors[correct])

    def typo(self, string):
        # Replace one character with a typo.
        if not string:
            return string
        i = random.randint(0, len(string)-1)
        incorrect = self.find_typo(string[i])
        return string[:i] + incorrect + string[i+1:]

    def typo_add(self, string):
        # Add a typo character hit before or after the correct character.
        if not string:
            return string
        i = random.randint(0, len(string)-1)
        correct = string[i]
        incorrect = self.find_typo(correct)
        if random.random() < 0.5:
            incorrect = correct + incorrect
        else:
            incorrect = incorrect + correct
        return string[:i] + incorrect + string[i+1:]

    def transpose(self, string):
        # Transpose two characters.
        if len(string) < 2:
            return string
        i = random.randint(0, len(string)-2)
        return string[:i] + string[i+1] + string[i] + string[i+2:]

    def duplicate(self, string):
        # Duplicate a character.
        if not string:
            return string
        i = random.randint(0, len(string)-1)
        return string[:i] + string[i] + string[i] + string[i+1:]

    def delete(self, string):
        # Delete a character.
        if not string:
            return string
        i = random.randint(0, len(string)-1)
        return string[:i] + string[i+1:]

    def delete_space(self, string):
        poses = [i for i, c in enumerate(string) if c == ' ']
        if not poses:
            return string
        i = random.choice(poses)
        return string[:i] + string[i+1:]

    def uppercase_word(self, string):
        words = string.split(" ")
        if not words:
            return string
        i = random.randint(0, len(words)-1)
        return " ".join(words[:i] + [words[i].upper()] + words[i+1:])

    def lowercase_uppercase_letter(self, string):
        poses = [(i,c) for i, c in enumerate(string) if c.upper() == c]
        i,c = random.choice(poses)
        return string[:i] + c.lower() + string[i+1:]

    def uppercase_letter(self, string):
        if not string:
            return string
        i = random.randint(0, len(string)-1)
        return string[:i] + string[i].upper() + string[i+1:]

    def uppercase_entire_string(self, string):
        return string.upper()

    def lowercase_entire_string(self, string):
        return string.upper()
    
    def remove_word(self, string):
        words = string.split(" ")
        if not words:
            return string
        i = random.randint(0, len(words)-1)
        return " ".join(words[:i] + words[i+1:])

    def omit_period(self, string):
        if not string:
            return string
        if string[-1] == '.':
            return string[:-1]
        return string

    def extra_space_at_beginning(self, string):
        return u"\N{EN SPACE}" + string

    def type(self, correct, so_far=""):
        length = len(correct)
        s = str(correct)
        transforms_per_100 = random.gauss(
            self.mean_transforms, self.stdev_transforms
        )
        chunks_100 = max(1, length/100)
        total_transforms = max(1, transforms_per_100 * chunks_100)

        for i in range(int(total_transforms)):
            s = self.wmt.choice()(s)
        s = so_far + s
        if random.random() < 0.1 and len(s) < 100:
            # If it's short, we may type the whole thing again.
            s = self.type(correct, s + " ")
        return s
